﻿-----------------------------------------------------------------------------
--
-- data
--   \aura
--     \<playerName>
--       - count
--       \auraList
--         \<auraName>
--		     - count, isBuff
--           \times
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentAura")
L:RegisterTranslations("enUS", function() return {
	["Aura Recipient List"] = true,
	["Auras"] = true,
	["Aura Details"] = true,
	["Aura Times"] = true,
	["Buff/Debuff"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Aura Recipient List"] = "버프 수령인 목록",
	["Auras"] = "버프",
	["Aura Details"] = "버프 정보",
	["Aura Times"] = "버프 시간",
} end)

-----------------------------------------------------------------------------

local AssessmentAura = Assessment:NewModule("Aura", "AceEvent-2.0", "Parser-3.0")
AssessmentAura.db = Assessment:AcquireDBNamespace("Aura")
Assessment:RegisterDefaults("Aura", "profile", {
	enable = true,
	buffColor	= { 0.3, 0.8, 0.3 },
	debuffColor = { 0.8, 0.3, 0.3 },
})

-----------------------------------------------------------------------------

local profile
local emptyMethod = Assessment.EmptyMethod
local new = Assessment.NewTable

-----------------------------------------------------------------------------

Assessment.TagData.AuraCount =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local aura = fightData.aura
				if not aura then return 0 end
				
				local playerAura = aura[data.key]
				if not playerAura then return 0 end

				return playerAura.count
			 end,
	data = "aura",
	guaranteeResult = true,
	numeric = true,
}

-----------------------------------------------------------------------------

Assessment.ListType.AuraRecipients =
{
	method = function(fightData)
				if not fightData.aura then return emptyMethod end
				return pairs(fightData.aura)
			 end,
	data = "aura",
	description = L["Aura Recipient List"],
}

-----------------------------------------------------------------------------

Assessment.ColorType.BuffDebuff = 
{
	method = function(fightData, data)
				return data.value.isBuff and profile.buffColor or profile.debuffColor
			 end,
	description = L["Buff/Debuff"]
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.AuraTimes =
{
	titleTag			= ("%s: [WindowParameter(1)]"):format(L["Aura Times"]),
	listType			= "EnumerateTable",
	listDataTag			= "aura,[WindowParameter(1)],auraList,[WindowParameter(2)],times",
	barValueTag			= "0",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { count = 2, { tag = "[WindowParameter(2)] [Rank]" }, { tag = "[Value:FormatTime]" } },
	description			= L["Aura Times"],
	selectable			= false,
	favorite			= false,
	defaultColor		= { 0.8, 0.8, 0.5 },
}

Assessment.DefaultDisplaySets.AuraDetails =
{
	titleTag			= ("%s: [WindowParameter(1)]"):format(L["Aura Details"]),
	listType			= "EnumerateTable",
	listDataTag			= "aura,[WindowParameter(1)],auraList",
	barValueTag			= "Count",
	filterTag			= "1",
	fightIndex			= -1,
	text				= { count = 2, { tag = "[Rank]. [Key]" }, { tag = "[BarValue]" } },
	description			= L["Aura Details"],
	selectable			= false,
	colorTypeList		= { "BuffDebuff", count = 1 },
	defaultColor		= { 0.8, 0.8, 0.5 },
	favorite			= false,
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Aura Times"],
	onClickData			= "[WindowParameter(1)],[Key]",
}

Assessment.DefaultDisplaySets.Auras =
{
	titleTag			= L["Auras"],
	listType			= "GroupMembers", 
	barValueTag			= "AuraCount", 
	filterTag			= "BarValue ~= 0",
	fightIndex			= 1,
	description			= L["Auras"],
	defaultColor		= { 0.8, 0.8, 0.5 },
	colorTypeList		= { "Class", count = 1 },
	onClickType			= "SHOW_DISPLAY_SET",
	onClickDisplaySet	= L["Aura Details"],
	onClickData			= "[Key]",
	favorite			= false,
}

-----------------------------------------------------------------------------

Assessment.ListType.EnumerateTable.updateData.aura = "aura"

-----------------------------------------------------------------------------

function AssessmentAura:OnInitialize()
	profile = self.db.profile
end

function AssessmentAura:OnEnable()
	self:RegisterParserEvent({ eventType = "Aura" }, "OnParserAuraEvent")
end

function AssessmentAura:OnDisable()
end

-----------------------------------------------------------------------------

function AssessmentAura:OnParserAuraEvent(eventData)
	local auraTime = time()
	local time = GetTime()

	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.aura = time
		local aura = data.aura
		if not aura then
			aura = new()
			data.aura = aura
		end

		local playerAura = aura[eventData.recipientName]
		local auraList
		if playerAura then
			playerAura.count = playerAura.count + 1
			auraList = playerAura.auraList
		else
			playerAura = new()
			auraList = new()
			playerAura.count = 1
			playerAura.auraList = auraList
			aura[eventData.recipientName] = playerAura
		end

		local auraData = auraList[eventData.abilityName]
		if auraData then
			auraData.count = auraData.count + 1
			local times = auraData.times
			times[#times+1] = auraTime
		else
			auraData = new()
			local times = new()
			auraData.count = 1
			auraData.times = times
			auraData.isBuff = eventData.isBuff
			times[#times+1] = auraTime
			auraList[eventData.abilityName] = auraData
		end
	end
end

-----------------------------------------------------------------------------
